#image_box {
    float:left;
    width:402px;
}

#image_thumbnail {
    border:1px solid #336699;
    margin-bottom:5px;
}

#image_caption {

}

#image_copyright {
    font-style: italic;
    text-align:right;
}

#metadata_box {
    margin-left:450px;
}

.metadata_key {
    background-color:#a0c4d5;
    border-bottom:1px solid #336699;
    border-left:1px solid #336699;
    color:#222;
    font-weight:bold;
}

.metadata_value {
    border-bottom:1px solid #336699;
    border-left:1px solid #336699;
    border-right:1px solid #336699;
}
